package cn.kdan.cloud.pdf.office.account.service.impl;

import cn.kdan.cloud.pdf.office.account.constant.UserConstant;
import cn.kdan.cloud.pdf.office.account.model.*;
import cn.kdan.cloud.pdf.office.account.service.CompanyService;
import cn.kdan.cloud.pdf.office.account.service.UserService;
import cn.kdan.cloud.pdf.office.account.service.UserSubscriptionInfoService;
import cn.kdan.cloud.pdf.office.account.utils.RSAUtils;
import cn.kdan.cloud.pdf.office.api.account.vo.UserDetailVO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserPageVO;
import cn.kdan.cloud.pdf.office.api.account.dto.*;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.ValidStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.AccountTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PlatformTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.RedisUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.account.mapper.ext.ExtUserMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.*;

@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private ExtUserMapper userMapper;
    @Autowired
    private CompanyService companyService;
    @Autowired
    private UserSubscriptionInfoService userSubscriptionInfoService;

    @Autowired
    private RedisUtils<String, Object> redisUtils;


    @Override
    public UserVO getById(String userId) {
        return convert(userMapper.selectByPrimaryKey(userId));
    }

    @Override
    public User getUser(String userId) {
        return userMapper.selectByPrimaryKey(userId);
    }

    @Override
    public PageInfo<UserPageVO> page(Integer page, Integer pageSize,
                                     String selectType,
                                     List<String> appIds,
                                     List<String> roles,
                                     List<String> accountTypes,
                                     List<String> accountSources,
                                     List<String> payTypes,
                                     String queryTimeType,
                                     Date startTime,
                                     Date endTime,
                                     List<String> queryString) {
        PageHelper.startPage(page, pageSize);
        return new PageInfo<>(userMapper.listWithInfo(selectType,appIds,roles,accountTypes,accountSources,payTypes,queryTimeType,startTime,endTime,queryString));
    }




    @Override
    public User getByAccount(String account, String appId, String platformType) {
        List<User> list = userMapper.getByAccount(account,appId,Integer.parseInt(platformType));
        return CollectionUtils.isEmpty(list) ? null : list.get(0);
    }

    @Override
    public UserVO getByAppAccount(String account, String appId, String platformType) {
        List<User> list = userMapper.getByAccount(account,appId,Integer.parseInt(platformType));
        return CollectionUtils.isEmpty(list) ? null : convert(list.get(0));
    }


    public UserVO convert(User user) {
        if(user == null){
            return new UserVO();
        }
        UserVO userVO = new UserVO();
        BeanUtils.copyProperties(user, userVO);
        return userVO;
    }

    @Override
    public UserPageVO getUserForTransfer(String userId) {
       return userMapper.getUserByIdWithApp(userId);
    }

    @Override
    public UserDetailVO detail(String userId) {
        UserDetailVO detailVO = new UserDetailVO();
        User user = userMapper.selectByPrimaryKey(userId);
        detailVO.setId(user.getId());
        detailVO.setEmail(user.getEmail());
        if(StringUtils.isNotEmpty(user.getCompanyId())){
          VppCompany company =   companyService.getById(user.getCompanyId());
          detailVO.setCompanyName(company.getName());
        }
        //todo 设置产品信息
        return detailVO;
    }

    @Override
    public void updateUserForOrder(UpdateUserForOrderDTO updateUserForOrderDTO) {
        //支付成功之后修改用户状态
        User user = userMapper.selectByPrimaryKey(updateUserForOrderDTO.getUserId());
        //通过用户可以查到对应的app
        user.setUpdatedAt(new Date());
        //账号类型 付费账号1 免费账号0 AccountTypeEnum
        user.setAccountType(AccountTypeEnum.PAID_ACCOUNT.value());
        //通过用户id和平台类型查询会员信息表，如果有会员信息则更新，如果没有会员信息就插入
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoService.getByUserIdAndPlatform(user.getId(), updateUserForOrderDTO.getPlatform());
        if(ObjectUtils.isEmpty(userSubscriptionInfo)){
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    .withUserId(user.getId()).withStatus(updateUserForOrderDTO.getUserStatus())
                    .withPayType(updateUserForOrderDTO.getPayTpe()).withEndDate(updateUserForOrderDTO.getEndDate());
            userSubscriptionInfoService.insert(newInfo);
        }else{
          //更新
            userSubscriptionInfo.setPayType(updateUserForOrderDTO.getPayTpe());
            userSubscriptionInfo.setStatus(updateUserForOrderDTO.getUserStatus());
            userSubscriptionInfo.setEndDate(updateUserForOrderDTO.getEndDate());
            userSubscriptionInfoService.update(userSubscriptionInfo);
        }
        userMapper.updateByPrimaryKey(user);
    }


    @Override
    public void register(UserRegisterDTO userRegisterDTO) {
        //参数校验
        validRegisterParam(userRegisterDTO);
        User user = initUser(userRegisterDTO.getAppId(),userRegisterDTO.getUsername(),userRegisterDTO.getPassword());
        //todo 发邮件
    }

    private User initUser(String appId, String email, String pwd) {
        User user = new User();
        user.setId(CommonUtils.generateId());
        user.setAppId(appId);
        user.setEmail(email);
        user.setCreatedAt(new Date());
        user.setAccountType(AccountTypeEnum.FREE_ACCOUNT.value());
        user.setPlatformType(PlatformTypeEnum.WEBSITE.value());
        user.setValidFlag(ValidStatusEnum.VALID.value());
        if(StringUtils.isNotEmpty(pwd)){
            user.setDigestPassword(new RSAUtils().encrypt(pwd));
        }
        userMapper.insert(user);
        return user;
    }

    @Override
    public void permissionGift(PermissionGiftDTO permissionGiftDTO) {
        String userId = "";
        User user = getByAccount(permissionGiftDTO.getAppId(),permissionGiftDTO.getEmail(), PlatformTypeEnum.WEBSITE.value().toString());
        //如果用户不存在
        if (ObjectUtils.isEmpty(getByAppAccount(permissionGiftDTO.getAppId(),permissionGiftDTO.getEmail(), PlatformTypeEnum.WEBSITE.value().toString()))) {
            user = initUser(permissionGiftDTO.getAppId(),permissionGiftDTO.getEmail(),"");
        }
        //todo 给指定用户用户创建一个订单
    }

    @Override
    public void permissionTransfer(PermissionTransferDTO permissionTransferDTO) {
        //修改用户的email
        User user = getUser(permissionTransferDTO.getOldUserId());
        user.setEmail(permissionTransferDTO.getNewEmail());
        user.setUpdatedAt(new Date());
        userMapper.updateByPrimaryKey(user);
        //todo 修改用户的订阅(填充remark)

    }

    @Override
    public void logOff(LogOffUserDTO logOffUserDTO) {
        User user = getUserWithCheck(logOffUserDTO.getUserId());
        handleLogOff(user,logOffUserDTO.getRemark());
    }

    @Override
    public void updateUser(User user) {
      userMapper.updateByPrimaryKeySelective(user);
    }

    @Override
    public void logOffForUser(String code, String userId) {
        User user = getUser(userId);
        //检查验证码是否正确
        checkEmailCodeValid(EmailCodeTypeEnum.USER_REGISTER, user.getEmail(), code,user.getAppId());
        //注销用户 todo 注销中 三天时间删除
        handleLogOff(user,"用户手动注销");
    }

    @Override
    public UserInfoVO getInfoById(String id) {
        UserInfoVO infoVO = new UserInfoVO();
        User user = getUser(id);
        BeanUtils.copyProperties(user,infoVO);
        List<UserSubscriptionInfo> list = userSubscriptionInfoService.listByUserId(id);
        List<UserSubscriptionInfoVO> voList = new ArrayList<>();
        for (UserSubscriptionInfo userSubscriptionInfo : list) {
            UserSubscriptionInfoVO vo = new UserSubscriptionInfoVO();
            BeanUtils.copyProperties(userSubscriptionInfo,vo);
            voList.add(vo);
        }
        infoVO.setSubscriptionInfoList(voList);
        return infoVO;
    }

    /**
     * 注销公共方法
     *
     * @param user 用户
     */
    private void handleLogOff(User user,String remark) {
        user.setRemark(remark);
        user.setUpdatedAt(new Date());
        user.setValidFlag(PDFOfficeUserStatusEnum.LOGGED_OUT.value().toString());
        userMapper.updateByPrimaryKeySelective(user);
    }

    private User getUserWithCheck(String userId) {
        User user = userMapper.selectByPrimaryKey(userId);
        if(ObjectUtils.isEmpty(user)){
            throw new BackendRuntimeException(UserConstant.EXCEPTION_MSG_USER_NOT_EXIST);
        }
        return user;
    }

    private void validRegisterParam(UserRegisterDTO param) {
        //邮箱是否被注册
        if (!ObjectUtils.isEmpty(getByAppAccount(param.getAppId(),param.getUsername(),param.getPlatformType()))) {
            throw new BackendRuntimeException(UserConstant.EXCEPTION_MSG_EMAIL_EXIST);
        }
        //验证邮箱格式
        String emailRegex = CommonConstant.emailRegex;
        if (!param.getUsername().matches(emailRegex)) {
            throw new BackendRuntimeException(UserConstant.EXCEPTION_MSG_EMAIL_INVALID);
        }
        //邮件验证码错误
        checkEmailCodeValid(EmailCodeTypeEnum.USER_REGISTER, param.getUsername(), param.getVerifyCode(), param.getAppId());
    }



    private void checkEmailCodeValid(EmailCodeTypeEnum type, String account, String code, String appId) {
        //获取用户存在redis中的登录邮箱验证码
        String captchaCode = redisUtils.hget(CommonConstant.EMAIL_VERIFY_CODE_KEY + type.value()  + CommonConstant.STRIKE_THROUGH + appId , account);
        if(StringUtils.isNotEmpty(code) && code.equals(captchaCode)){
            //验证通过删除
            redisUtils.hdel(CommonConstant.EMAIL_VERIFY_CODE_KEY + type.value() + CommonConstant.STRIKE_THROUGH + appId  , account);
        }else{
            throw new BackendRuntimeException(CommonConstant.EMAIL_VERIFY_CODE_KEY_ERROR);
        }
    }




}
