package cn.kdan.cloud.pdf.office.api.account.feign;

import cn.kdan.cloud.pdf.office.api.account.feign.hystrix.LoginDeviceApiHystrix;
import cn.kdan.cloud.pdf.office.api.account.vo.LoginDevice;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;


/**
 * @author tangxiangan
 */
@Component
@FeignClient(value = CommonConstant.ACCOUNT_PROVIDER_NAME + "/loginDevice", fallback = LoginDeviceApiHystrix.class)
public interface LoginDeviceApi {


    @PostMapping(value = "create")
    ResultMap<Boolean> create(@RequestBody LoginDevice loginDevice);

    /**
     * 删除登录设备记录
     * @param deviceSign
     * @param userId
     * @return
     */
    @PostMapping("delete")
    ResultMap<Boolean> delete(@RequestParam("deviceSign") String deviceSign, @RequestParam("userId") String userId) ;


    /**
     * 获取用户的设备登录情况
     * @param userId
     * @return
     */
    @GetMapping("getByUserId")
    ResultMap<List<LoginDevice>> getByUserId(@RequestParam("userId") String userId);


    /**
     * 获取登录设备记录
     * @param deviceSign
     * @param userId
     * @return
     */
    @GetMapping("getByDeviceSignAndAppId")
    ResultMap<LoginDevice> getByDeviceSignAndAppId(@RequestParam("deviceSign") String deviceSign, @RequestParam("userId") String userId) ;


}
