package cn.kdan.cloud.pdf.office.api.account.feign;

import cn.kdan.cloud.pdf.office.api.account.dto.*;
import cn.kdan.cloud.pdf.office.api.account.vo.UserDetailVO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserPageVO;
import cn.kdan.cloud.pdf.office.common.config.FeignConfig;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import com.github.pagehelper.PageInfo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Date;
import java.util.List;


@Component
@FeignClient(value = CommonConstant.ACCOUNT_PROVIDER_NAME + "/user" , configuration = FeignConfig.class)
public interface UserApi {


    @GetMapping(value = "getByAccount")
    ResultMap<UserVO> getByAccount(@RequestParam("account") String account);


    @GetMapping(value = "getByAppAccount")
    ResultMap<UserVO> getByAppAccount(@RequestParam("account") String account, @RequestParam("appId") String appId, @RequestParam("platformType") String platformType);

    @GetMapping(value = "getById")
    ResultMap<UserVO> getById(@RequestParam("id") String id);

    /**
     * 用户信息包括用户的会员信息
     *
     * @param id id
     * @return UserInfoVO
     */
    @GetMapping("getInfoById")
    ResultMap<UserInfoVO> getInfoById(@RequestParam("id") String id);

    @PostMapping(value = "register")
    ResultMap<Boolean> register(@RequestBody UserRegisterDTO userRegisterDTO);

    /**
     * 更新
     *
     * @param userVO
     */
    @PostMapping("/updateUser")
    ResultMap<Boolean> updateUser(@RequestBody UserVO userVO);

    /**
     * 支付成功之后修改用户状态
     *
     * @param updateUserForOrderDTO dto
     */
    @PostMapping(value = "updateUserForOrder")
    ResultMap<Boolean> updateUserForOrder(@RequestBody UpdateUserForOrderDTO updateUserForOrderDTO);


    @GetMapping("/page")
    ResultMap<PageInfo<UserPageVO>> page(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                @RequestParam(required = false) String selectType,
                                                @RequestParam(required = false) List<String> appIds,
                                                @RequestParam(required = false)  List<String> roles,
                                                @RequestParam(required = false)  List<String> accountTypes,
                                                @RequestParam(required = false)  List<String> accountSources,
                                                @RequestParam(required = false)  List<String> payTypes,
                                                @RequestParam(required = false)  String queryTimeType,
                                                @RequestParam(required = false) Date startTime,
                                                @RequestParam(required = false)  Date endTime,
                                                @RequestParam(required = false)  List<String> queryString);
    /**
     * 运营后台查看账号详情
     *
     * @param userId 用户id
     * @return ResultMap
     */
    @GetMapping("/detail")
    ResultMap<UserDetailVO> detail(@RequestParam("userId") String userId);


    /**
     * 权限赠送
     *
     * @param permissionGiftDTO permissionGiftDTO
     * @return Boolean
     */
    @PostMapping("/permissionGift")
    ResultMap<Boolean> permissionGift(@RequestBody PermissionGiftDTO permissionGiftDTO);

    /**
     * 权限转移
     *
     * @param permissionTransferDTO permissionTransferDTO
     * @return Boolean
     */
    @PostMapping("/permissionTransfer")
    ResultMap<Boolean> permissionTransfer(@RequestBody PermissionTransferDTO permissionTransferDTO);



    /**
     * 后台强制注销用户
     *
     * @param logOffUserDTO
     */
    @PostMapping("/logOff")
    ResultMap<Boolean> logOff(@RequestBody LogOffUserDTO logOffUserDTO);


    /**
     * 用户自己手动注销
     *
     * @param code
     * @return
     */
    @PostMapping("/logOffForUser")
    ResultMap<Boolean> logOffForUser(@RequestParam("code") String code,@RequestParam("userId")  String userId);
}
