package cn.kdan.cloud.pdf.office.api.payment.feign;

import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.feign.hystrix.OrderHystrix;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreatOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.NotNull;
import java.util.List;


@Component
@FeignClient(value = CommonConstant.PAYMENT_PROVIDER_NAME + "/order", fallback = OrderHystrix.class)
public interface OrderApi {

    /**
     * 创建支付订单
     *
     * @param creatOrderDTO creatOrderDTO
     * @return CreatOrderResultVO
     */
    @PostMapping("/creatOrder")
    ResultMap<CreatOrderResultVO> creatOrder(@Validated @RequestBody CreatOrderDTO creatOrderDTO);

    /**
     * 手动创建虚拟订单
     *
     * @param creatOrderManual creatOrderManual
     * @return OrderUpdateVO
     */
    @PostMapping("/creatOrderManual")
    ResultMap<OrderUpdateVO> creatOrderManual(@Validated @RequestBody CreatOrderManualDTO creatOrderManual);

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    @GetMapping("/getOrderById")
    ResultMap<OrdersVO> getOrderById(@Validated @NotNull(message = "orderId不能为空")
                                     @RequestParam("orderId") String orderId);

    /**
     * 通过自定义订单id获取订单信息
     *
     * @param tradeNo tradeNo
     * @return Orders
     */
    @GetMapping("/getOrderByTradeNo")
    ResultMap<List<OrdersVO>> getOrderByTradeNo(@Validated @NotNull(message = "tradeNo不能为空")
                                          @RequestParam("tradeNo") String tradeNo);

    /**
     * 手动推单
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     * @return Void
     */
    @PostMapping("/chargebackOrder")
    ResultMap<Void> chargebackOrder(@RequestBody ChargebackOrderDTO chargebackOrderDTO);
}
