package cn.kdan.cloud.pdf.office.api.payment.feign;

import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.feign.hystrix.SubscriptionHystrix;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.NotNull;


@Component
@FeignClient(value = CommonConstant.PAYMENT_PROVIDER_NAME + "/subscription", fallback = SubscriptionHystrix.class)
public interface SubscriptionApi {
    /**
     * 创建订阅信息
     *
     * @param subscriptionInfo subscriptionInfo
     * @return 订阅id
     */
    @PostMapping("/creatSubscription")
    ResultMap<String> creatSubscription(@Validated @RequestBody CreatSubscriptionDTO subscriptionInfo);

    /**
     * 通过用户id和产品id 获取最新的订阅信息
     *
     * @param userId   userId
     * @param platform platform
     * @return Subscriptions
     */
    @GetMapping("/getSubscriptions")
    ResultMap<SubscriptionsVO> getSubscriptions(@Validated @NotNull(message = "userId 不能为空") @RequestParam("userId") String userId,
                                                @Validated @NotNull(message = "platform 不能为空") @RequestParam("platform") Integer platform);

    /**
     * 手动创建订阅信息
     *
     * @param subscriptionManual subscriptionManual
     * @return 订阅id
     */
    @PostMapping("/giveSubscriptionManual")
    ResultMap<String> giveSubscriptionManual(@Validated @RequestBody CreatSubscriptionManualDTO subscriptionManual);


    /**
     * 查询当前用户的订阅计划和支付订单
     *
     * @param userId   userId
     * @param platform platform
     * @return SubscriptionSchemeVO
     */
    @GetMapping("/getSubscriptionSchemeInfo")
    ResultMap<SubscriptionSchemeVO> getSubscriptionSchemeInfo(@NotNull(message = "用户id不能为空")@RequestParam("userId") String userId,
                                                              @NotNull(message = "platform不能为空")@RequestParam("platform") Integer platform);

}
