package cn.kdan.cloud.pdf.office.api.product.feign;

import cn.kdan.cloud.pdf.office.api.product.feign.hystrix.ProductHystrix;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductPriceVo;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.NotNull;
import java.util.List;


@Component
@FeignClient(value = CommonConstant.PRODUCT_PROVIDER_NAME + "/product", fallback = ProductHystrix.class)
public interface ProductApi {


    @GetMapping("/getProductPrice")
    ResultMap<ProductPriceVo> getProductPrice(@NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                              @NotNull(message = "userId不能为空") @RequestParam("userId") String userId);


    @GetMapping("/getProduct")
    ResultMap<ProductVO> getProduct(@NotNull(message = "产品id不能为空") @RequestParam("productId") String productId);


    @GetMapping("/getProductList")
    ResultMap<IPage<ProductVO>> getProductList(@RequestParam("size") Long size, @RequestParam("page") Long page,
                                               @RequestParam("appId") String appId);

    /**
     * 获取新用户专用活动
     *
     * @param appId appId
     * @return 新用户专用活动
     */
    @GetMapping("/getNewUserProduct")
    ResultMap<List<ProductVO>> getNewUserProduct(@RequestParam String appId);

}
