package cn.kdan.cloud.pdf.office.common.utils;

import cn.kdan.cloud.pdf.office.common.enums.MyDateCompany;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.exception.MyDateUtilsErrorMessage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.Date;

/**
 * @author ComPDFKit-WPH 2023/1/30
 * <p>
 * 时间工具类
 */
public class MyDateUtils {

    private final static ZoneId zoneId = ZoneId.systemDefault();

    private MyDateUtils() {
    }

    /**
     * 获取 一个自定义发票编号/??
     *
     * @return 发票编号/??
     */
    public static String getTimeStamp() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        return simpleDateFormat.format(new Date());
    }


    /**
     * date 转 LocalDateTime
     *
     * @param date date
     * @return LocalDateTime
     */
    public static LocalDateTime dateToLocalDateTime(Date date) {
        return LocalDateTime.ofInstant(date.toInstant(), zoneId);
    }

    /**
     * date 转 LocalDateTime
     *
     * @param date LocalDateTime
     * @return Date
     */
    public static Date localDateTimeToDate(LocalDateTime date) {
        return Date.from(date.atZone(zoneId).toInstant());
    }

    /**
     * 时间 加法
     *
     * @param dateTime  dateTime
     * @param forPeriod 1.years, 1.months
     * @return LocalDateTime
     */
    public static LocalDateTime TimeAddition(LocalDateTime dateTime, String forPeriod) {
        int index = forPeriod.trim().indexOf(".");
        if (index < 1 || index > (forPeriod.trim().length() - 1)){
            throw new BackendRuntimeException(MyDateUtilsErrorMessage.DATA_ERROR);
        }
        int duration = Integer.parseInt(forPeriod.trim().substring(0, index));
        String company = forPeriod.trim().substring(index + 1);
        MyDateCompany myDateCompany = MyDateCompany.valueOf(company);
        if (ObjectUtils.isEmpty(myDateCompany)){
            throw new BackendRuntimeException(MyDateUtilsErrorMessage.DATA_ERROR);
        }
        return dateTime.plus(duration, myDateCompany.getChronoUnit());
    }


}
