package cn.kdan.cloud.pdf.office.payment;

import cn.kdan.cloud.pdf.office.api.account.annotation.EnableAccountApiClient;
import cn.kdan.cloud.pdf.office.api.product.annotation.EnableProductApiClient;
import cn.kdan.cloud.pdf.office.common.annotation.*;
import cn.kdan.cloud.pdf.office.payment.properties.PaddleProperties;
import cn.kdan.cloud.pdf.office.payment.properties.PaypalProperties;
import cn.kdan.cloud.pdf.office.payment.verifier.Verifier;
import lombok.RequiredArgsConstructor;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.netflix.hystrix.EnableHystrix;
import org.springframework.context.annotation.Bean;

/**
 * @author ComPDFKit-WPH 2023/1/12
 */

@SpringBootApplication
@EnableDiscoveryClient
@EnableHystrix
@EnableConfigurationProperties({PaypalProperties.class, PaddleProperties.class})
@EnableMybatisPlusConfig
@MapperScan("cn.kdan.cloud.pdf.office.payment.mapper")
@EnableMyRabbitMqHandler
@RequiredArgsConstructor
@EnableCaching
@EnableCommonTools
@EnableProductApiClient
@EnableAccountApiClient
@EnableCommonFeignExceptionHandle
@EnableControllerExceptionHandler
public class PDFOfficePaymentApplication {

    public static void main(String[] args) {
        SpringApplication.run(PDFOfficePaymentApplication.class, args);
    }

    @Bean
    public Verifier getVerifier(PaddleProperties properties) {
        return new Verifier("-----BEGIN PUBLIC KEY-----\n" +
                "MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEA5cPUUYfdeOwkBpMM4iI9\n" +
                "TO79SDZCZwRdUTkeH3WpbTqWXWUoRQZTlsnGS0yeUU7hExDjCrWCW7pAw3YHIYJJ\n" +
                "DsiERHd+3RoCRnQAYOWxMJ5LGeaEOZzj66Ng6RS5FqVh8kPL+QTwyqKWZfIamw1T\n" +
                "xsUBcSq5I8QB9A3ugH5UUnAK/VrG8EV8hLnUYdu5OtUusZ/HggRgKR4+IJ79h/5x\n" +
                "YrjFgUsgWyuyG6slxLomXKt2MDwGPmxntumxbN5LEXIkOA/2d+M1wfoQj6hAwxD5\n" +
                "ewI4+YJiHZ5F1rP+cBAGPDaYh/Fg1b4b1SSzpz7Nvn0gbDluori1EC7Sf8+VlrTC\n" +
                "dJ7cEHqTR+LjzO3apPfs46+M36g3YPGVZ3vQtfFoxh+SEKYfXRp9NNoUCYAHOsDv\n" +
                "1cZ5tZYAqSIhQjhZt9BrAexNHy/rcfs9wow3Lf19U0QSuThBJwbnsVI0CAhNTona\n" +
                "4LL1E35yNESBipArhSNF8Gmz1z+GMS3CQ3jyqy6XXzkaS9WFpJNmP67aqekKsfyh\n" +
                "p0Xv0NK4PXgQ75lQ1aQuzIOoAmvPLw5VFkvr3lXug62vxXNujM9M+5YRghx98Vxx\n" +
                "kjUxQmGQOvRKeZ55ebSX9JQgqXdZejnxhuYyc+RFWu7Q1jMu5ecD26ld82YYRsgO\n" +
                "9JZbO8CXGCyAltQ39TRJ9uECAwEAAQ==\n" +
                "-----END PUBLIC KEY-----");
    }


}
