package cn.kdan.cloud.pdf.office.payment.client;

import cn.kdan.cloud.pdf.office.api.payment.bo.CreatSubscriptionsPaypalBO;
import cn.kdan.cloud.pdf.office.api.payment.bo.CreatSubscriptionsResultPaypalBO;
import cn.kdan.cloud.pdf.office.api.payment.constant.PaypalAPIConstant;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.payment.interceptor.PaypalClientHttpRequestInterceptor;
import cn.kdan.cloud.pdf.office.payment.properties.PaypalProperties;
import org.jetbrains.annotations.NotNull;
import org.springframework.http.*;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.stereotype.Component;
import org.springframework.web.client.DefaultResponseErrorHandler;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/1/16
 * <p>
 * PaypalClient
 */
@Component
public class PaypalClient {

    private final RestTemplate restTemplate = new RestTemplate();

    private static String PAYPAL_HOST;

    public PaypalClient(PaypalProperties properties, PaypalClientHttpRequestInterceptor paypalClientHttpRequestInterceptor) {
        PAYPAL_HOST = properties.getAddress();
        SimpleClientHttpRequestFactory factory = new SimpleClientHttpRequestFactory();
        factory.setReadTimeout(properties.getReadTimeOut());
        factory.setConnectTimeout(properties.getConnectTimeout());
        restTemplate.setRequestFactory(factory);
        restTemplate.setInterceptors(Collections.singletonList(paypalClientHttpRequestInterceptor));
        restTemplate.setMessageConverters(getConverts());
        restTemplate.setErrorHandler(new DefaultResponseErrorHandler() {
            @Override
            public void handleError(@NotNull ClientHttpResponse clientHttpResponse) throws IOException {
                //只要重写此方法，不去抛出HttpClientErrorException异常即可
//                HttpStatus statusCode = clientHttpResponse.getStatusCode();
            }
        });
    }

    private List<HttpMessageConverter<?>> getConverts() {
        List<HttpMessageConverter<?>> messageConverters = new ArrayList<>();
        // String转换器
        StringHttpMessageConverter stringConvert = new StringHttpMessageConverter();
        List<MediaType> stringMediaTypes = new ArrayList<MediaType>() {{
            add(MediaType.TEXT_PLAIN);
            add(MediaType.TEXT_HTML);
            add(MediaType.APPLICATION_JSON);
        }};
        stringConvert.setSupportedMediaTypes(stringMediaTypes);
        messageConverters.add(stringConvert);
        return messageConverters;
    }

    /**
     * 创建订阅
     *
     * @param creatSubscriptions creatSubscriptions
     * @return CreatSubscriptionsResultBO
     */
    public CreatSubscriptionsResultPaypalBO creatSubscriptions(CreatSubscriptionsPaypalBO creatSubscriptions) {
        String resultJson = restTemplate.exchange(PAYPAL_HOST + PaypalAPIConstant.CREAT_SUBSCRIPTIONS,
                HttpMethod.POST,
                new HttpEntity<>(JsonUtils.getJsonString(creatSubscriptions), getJsonHttpHeaders()),
                String.class).getBody();
        return JsonUtils.jsonStringToBean(resultJson, CreatSubscriptionsResultPaypalBO.class);
    }


    /**
     * 获取订阅信息
     *
     * @param subscriptionId subscriptionId
     * @return CreatSubscriptionsResultBO
     */
    public CreatSubscriptionsResultPaypalBO getSubscriptionInfo(@NotNull String subscriptionId){
        String resultJson = restTemplate.exchange(PAYPAL_HOST + PaypalAPIConstant.GET_SUBSCRIPTIONS_INFO + subscriptionId,
                HttpMethod.GET,
                new HttpEntity<>(getJsonHttpHeaders()),
                String.class).getBody();
        return JsonUtils.jsonStringToBean(resultJson, CreatSubscriptionsResultPaypalBO.class);
    }


    /**
     * 创建 APPLICATION_JSON HttpHeaders
     *
     * @return HttpHeaders
     */
    @NotNull
    private HttpHeaders getJsonHttpHeaders() {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        return httpHeaders;
    }

}
