package cn.kdan.cloud.pdf.office.payment.controller;

import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreatOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.payment.service.OrdersService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/2/6
 * <p>
 * OrderController
 */
@RestController
@RequestMapping("/order")
@RequiredArgsConstructor
public class OrderController {

    private final OrdersService ordersService;

    private final SubscriptionsService subscriptionsService;

    /**
     * 创建支付订单
     *
     * @param creatOrderDTO creatOrderDTO
     * @return CreatOrderResultVO
     */
    @PostMapping("/creatOrder")
    public ResultMap<CreatOrderResultVO> creatOrder(@Validated @RequestBody CreatOrderDTO creatOrderDTO){
        return ResultMap.success(ordersService.creatOrder(creatOrderDTO));
    }

    /**
     * 手动创建虚拟订单
     *
     * @param creatOrderManual creatOrderManual
     * @return OrderUpdateVO
     */
    @PostMapping("/creatOrderManual")
    public ResultMap<OrderUpdateVO> creatOrderManual(@Validated @RequestBody CreatOrderManualDTO creatOrderManual){
        return ResultMap.success(ordersService.creatOrderManual(creatOrderManual));
    }

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    @GetMapping("/getOrderById")
    public ResultMap<OrdersVO> getOrderById(@Validated @NotNull(message = "orderId不能为空")
                                            @RequestParam("orderId") String orderId){
        return ResultMap.success(ordersService.getOrderById(orderId));
    }

    /**
     * 通过自定义订单id获取订单信息
     *
     * @param tradeNo tradeNo
     * @return Orders
     */
    @GetMapping("/getOrderByTradeNo")
    public ResultMap<List<OrdersVO>> getOrderByTradeNo(@Validated @NotNull(message = "tradeNo不能为空")
                                                 @RequestParam("tradeNo") String tradeNo){
        return ResultMap.success(ordersService.getOrderByTradeNo(tradeNo));
    }

    /**
     * 手动推单
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     * @return Void
     */
    @PostMapping("/chargebackOrder")
    public ResultMap<Void> chargebackOrder(@Validated @RequestBody ChargebackOrderDTO chargebackOrderDTO){
        // 修改 订单状态
        ordersService.chargebackOrder(chargebackOrderDTO);
        // 修改 订阅状态
        subscriptionsService.chargebackSubscription(chargebackOrderDTO);
        return ResultMap.success();
    }



}
