package cn.kdan.cloud.pdf.office.payment.controller;

import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

/**
 * @author ComPDFKit-WPH 2023/2/6
 * <p>
 * SubscriptionController
 */
@RestController
@RequestMapping("/subscription")
@RequiredArgsConstructor
public class SubscriptionController {

    private final SubscriptionsService subscriptionsService;

    /**
     * 创建订阅信息
     *
     * @param subscriptionInfo subscriptionInfo
     * @return 订阅id
     */
    @PostMapping("/creatSubscription")
    public ResultMap<String> creatSubscription(@Validated @RequestBody CreatSubscriptionDTO subscriptionInfo) {
        return ResultMap.success(subscriptionsService.creatSubscription(subscriptionInfo));
    }

    /**
     * 通过用户id和产品id 获取最新的订阅信息
     *
     * @param userId userId
     * @param platform  platform
     * @return Subscriptions
     */
    @GetMapping("/getSubscriptions")
    public ResultMap<SubscriptionsVO> getSubscriptions(@Validated @NotNull(message = "userId 不能为空") @RequestParam("userId") String userId,
                                                       @Validated @NotNull(message = "platform 不能为空") @RequestParam("platform") Integer platform) {
        return ResultMap.success(subscriptionsService.getSubscriptions(userId, PlatformEnum.getEnumByValue(platform)));
    }

    /**
     * 手动创建订阅信息 赠送
     *
     * @param subscriptionManual subscriptionManual
     * @return 订阅id
     */
    @PostMapping("/giveSubscriptionManual")
    public ResultMap<String> giveSubscriptionManual(@Validated @RequestBody CreatSubscriptionManualDTO subscriptionManual) {
        return ResultMap.success(subscriptionsService.giveSubscriptionManual(subscriptionManual));
    }

    /**
     * 查询当前用户的订阅计划和支付订单
     *
     * @param userId   userId
     * @param platform platform
     * @return SubscriptionSchemeVO
     */
    @GetMapping("/getSubscriptionSchemeInfo")
    public ResultMap<SubscriptionSchemeVO> getSubscriptionSchemeInfo(@Validated @NotNull(message = "用户id不能为空") @RequestParam String userId,
                                                                     @Validated @NotNull(message = "platform不能为空") @RequestParam Integer platform) {
        return ResultMap.success(subscriptionsService.getSubscriptionSchemeInfo(userId, PlatformEnum.getEnumByValue(platform)));
    }

}
