package cn.kdan.cloud.pdf.office.payment.service;

import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.core.metadata.IPage;

/**
 * @author ComPDFKit-WPH 2023-01-28
 * <p>
 * 生成支付订阅
 */
public interface SubscriptionsService extends IService<Subscription> {

    /**
     * 列表查询
     *
     * @param query 查询参数
     * @return 列表数据
     */
    IPage<Subscription> page(Subscription query);

    /**
     * 创建订阅信息
     *
     * @param subscriptionInfo subscriptionInfo
     * @return 订阅id
     */
    String creatSubscription(CreatSubscriptionDTO subscriptionInfo);

    /**
     * 通过用户id和平台类型 获取最新的订阅信息
     *
     * @param userId   userId
     * @param platform 平台类型
     * @return Subscriptions
     */
    SubscriptionsVO getSubscriptions(String userId, PlatformEnum platform);

    /**
     * 通过用户id和产品id 获取最新的订阅信息
     *
     * @param userId    userId
     * @param productId productId
     * @return Subscriptions
     */
    SubscriptionsVO getSubscriptionsByProductId(String userId, String productId);

    /**
     * 创建订阅信息 自动续订
     *
     * @param subscriptionManual subscriptionManual
     * @return 订阅id
     */
    String creatSubscriptionManual(CreatSubscriptionManualDTO subscriptionManual);

    /**
     * 手动创建订阅信息 赠送
     *
     * @param subscriptionManual subscriptionManual
     * @return 订阅id
     */
    String giveSubscriptionManual(CreatSubscriptionManualDTO subscriptionManual);

    /**
     * 取消订阅 退单
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     */
    void chargebackSubscription(ChargebackOrderDTO chargebackOrderDTO);

    /**
     * 查询当前用户的订阅计划和支付订单
     *
     * @param userId       userId
     * @param platformEnum platformEnum
     * @return SubscriptionSchemeVO
     */
    SubscriptionSchemeVO getSubscriptionSchemeInfo(String userId, PlatformEnum platformEnum);

    /**
     * 根据主键id获取
     *
     * @param subscriptionId subscriptionId
     * @return SubscriptionsVO
     */
    SubscriptionsVO getSubscriptionBySubscriptionId(String subscriptionId);
}