package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.payment.bo.GeneratePayLinkPaddleBO;
import cn.kdan.cloud.pdf.office.api.payment.bo.GeneratePayLinkResultPaddleBO;
import cn.kdan.cloud.pdf.office.api.payment.bo.OrderSucceededBO;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreatOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductPriceVo;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.constant.RedisKeyConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.client.PaddleClient;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.mapper.OrdersMapper;
import cn.kdan.cloud.pdf.office.payment.service.OrdersService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * @author ComPDFKit-WPH 2023-01-29
 * <p>
 * OrdersServiceImpl
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class OrdersServiceImpl extends ServiceImpl<OrdersMapper, Order> implements OrdersService {

    private final ProductApi productApi;

    private final PaddleClient paddleClient;

    private final RabbitTemplate rabbitTemplate;

//    private final SubscriptionsService subscriptionsService;

    @Override
    @Cacheable(value = RedisKeyConstant.CACHE_PAYMENT_CREAT_ORDER + "#3#h",
            key = "T(String).valueOf(#creatOrderDTO.userId)" +
                    ".concat('-').concat(#creatOrderDTO.productId)" +
                    ".concat('-').concat(#creatOrderDTO.paymentMethod.value)",
            cacheManager = "myRedisCacheManager", unless = "#result == null")
    public CreatOrderResultVO creatOrder(CreatOrderDTO creatOrderDTO) {
        log.info("creatOrder:{}", creatOrderDTO);
        Order orders = new Order();
        orders.setAppId(creatOrderDTO.getAppId());
        orders.setUserId(creatOrderDTO.getUserId());
        orders.setProductId(creatOrderDTO.getProductId());
        orders.setPayment(creatOrderDTO.getPaymentMethod().getValue());
        orders.setTradeNo(UUID.randomUUID().toString());
        orders.setStatus(OrderConstant.CREATED);
        orders.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
//        orders.setDiscount(creatOrderDTO.getDiscount());
        String readeNo = MyDateUtils.getTimeStamp() + "-" + creatOrderDTO.getProductId() + "-" + (int) ((Math.random() * 9 + 1) * 1000);
        orders.setTradeNo(readeNo);
        // 查询当前产品多少钱
        ProductPriceVo productPrice = productApi.getProductPrice(creatOrderDTO.getProductId(),creatOrderDTO.getUserId()).getResult();
        switch (creatOrderDTO.getPaymentMethod()) {
            case WXPAY:
                break;
            case ALIPAY:
                break;
            case PADDLE:
                BigDecimal price = productPrice.getPrice();
                price = priceHandle(creatOrderDTO, price, orders);
                GeneratePayLinkPaddleBO paddleBO = new GeneratePayLinkPaddleBO();
                paddleBO.setPrices(new String[]{"USD:" + price.toString()});
                paddleBO.setProductId(productPrice.getPaddleProductId());
                paddleBO.setPassthrough(readeNo);
                GeneratePayLinkResultPaddleBO resultPaddleBO = paddleClient.creatSubscriptions(paddleBO);
                orders.setPrice(price);
                orders.setPagePayUrl(resultPaddleBO.getUrl());
                break;
            case NO_NEED_TO_PAY:
                break;
            case PAYPAL:
                break;
            // 当前支付方式无效
            default:
                throw new BackendRuntimeException(ErrorMessage.PAYMENT_METHOD_INVALID);
        }
        this.baseMapper.insert(orders);
        CreatOrderResultVO creatOrderResultVO = new CreatOrderResultVO();
        BeanUtils.copyProperties(orders, creatOrderResultVO);
        // 订单关闭
        log.info("rabbitMq 发送延时队列，关闭订单：{}", orders.getId());
        rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_EXCHANGE,
                RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY,
                orders.getId(),
                (msg -> {
                    msg.getMessageProperties().setExpiration((60 * 1000) + "");
                    return msg;
                }));
        return creatOrderResultVO;
    }

    @Override
    public void closeOrder(String orderId) {
        OrdersVO orders = getOrderById(orderId);
        log.info("当前订单信息：{}", orders);
        if (orders.getStatus().equals(OrderConstant.CREATED)) {
            Order order = new Order();
            order.setStatus(OrderConstant.CLOSED);
            order.setId(orderId);
            this.baseMapper.updateById(order);
        }
    }

    @Override
    public OrderUpdateVO updateOrderSucceeded(OrderSucceededBO orderSucceeded) {
        Order orders = new Order();
        orders.setThirdOrderNo(orderSucceeded.getThirdTradeNo());
        orders.setStatus(OrderConstant.COMPLETED);
        this.baseMapper.update(orders, new LambdaQueryWrapper<Order>()
                .eq(Order::getTradeNo, orderSucceeded.getTradeNo()));
        Order orderTradeNo = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>()
                .select(Order::getId, Order::getUserId,Order::getProductId,Order::getAppId)
                .eq(Order::getTradeNo, orderSucceeded.getTradeNo()));
        return OrderUpdateVO.builder()
                .orderId(orderTradeNo.getId())
                .userId(orderTradeNo.getUserId())
                .productId(orders.getProductId())
                .appId(orders.getAppId())
                .build();
    }

    @Override
    public OrdersVO getOrderById(String orderId) {
        OrdersVO ordersVO = new OrdersVO();
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getId, orderId)), ordersVO);
        return ordersVO;
    }

    @Override
    public List<OrdersVO> getOrderByTradeNo(String tradeNo) {
        return this.baseMapper.selectOrdersByTradeNo(tradeNo);
    }


    @Override
    public OrderUpdateVO creatOrderManual(CreatOrderManualDTO creatOrderManual) {
        Order orders = new Order();
        orders.setAppId(creatOrderManual.getAppId());
        orders.setProductId(creatOrderManual.getProductId());
        orders.setUserId(creatOrderManual.getUserId());
        orders.setTradeNo(creatOrderManual.getTradeNo());
        orders.setThirdTradeNo(creatOrderManual.getThirdTradeNo());
        orders.setPrice(creatOrderManual.getPrice());
        orders.setPayment(creatOrderManual.getPaymentMethod().getValue());
        this.baseMapper.insert(orders);
        return OrderUpdateVO.builder()
                .userId(creatOrderManual.getUserId())
                .orderId(orders.getId())
                .build();
    }


    @Override
    public void chargebackOrder(@NotNull ChargebackOrderDTO chargebackOrderDTO) {
        log.info("手动退单修改状态：{}", chargebackOrderDTO);
        // 修改订单状态
        Order orders = new Order();
        orders.setId(chargebackOrderDTO.getOrderId());
        orders.setStatus(OrderConstant.REFUND);
        this.baseMapper.updateById(orders);
        log.info("手动退单修改状态成功：{}", chargebackOrderDTO);
    }

    /**
     * 对价格进行处理
     *
     * @param creatOrderDTO creatOrderDTO
     * @param price         price
     * @param orders        orders
     * @return 新价格
     */
    private BigDecimal priceHandle(CreatOrderDTO creatOrderDTO, BigDecimal price, Order orders) {
//        if (!ObjectUtils.isEmpty(creatOrderDTO.getFinalPrice())) {
//            price = creatOrderDTO.getFinalPrice();
//        }
        if (!ObjectUtils.isEmpty(creatOrderDTO.getCouponCode())) {
            String couponCode = creatOrderDTO.getCouponCode();
            // TODO 优惠价查询价格
            BigDecimal couponCodePrice = BigDecimal.ONE;
            price = price.subtract(couponCodePrice);
            orders.setReducedPrice(couponCodePrice);
        }
        return price;
    }


}
