package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.dto.UpdateUserForOrderDTO;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.bo.OrderSucceededBO;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.PaddleWebhookDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PaidTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.service.OrdersService;
import cn.kdan.cloud.pdf.office.payment.service.PaddleWebhookService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * @author ComPDFKit-WPH 2023/1/30
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class PaddleWebhookServiceImpl implements PaddleWebhookService {

    private final OrdersService ordersService;

    private final SubscriptionsService subscriptionsService;

    private final UserApi userApi;

    private final ProductApi productApi;

    private final RabbitTemplate rabbitTemplate;


    @Override
    public void subscriptionCreatSucceeded(PaddleWebhookDTO paddleWebhookDTO) {
        log.info("paddle 订阅创建已成功 paddle订单id：{}",paddleWebhookDTO.getCheckoutId());
        OrderSucceededBO orderSucceeded = OrderSucceededBO.builder()
                .paymentTax(paddleWebhookDTO.getPaymentTax())
                .paymentMethod(paddleWebhookDTO.getPaymentMethod())
                .email(paddleWebhookDTO.getEmail())
                .thirdTradeNo(paddleWebhookDTO.getCheckoutId())
                .tradeNo(paddleWebhookDTO.getPassthrough())
                .build();
        // 修改订单状态
        OrderUpdateVO orderUpdate = ordersService.updateOrderSucceeded(orderSucceeded);
        // 添加订阅信息
        CreatSubscriptionDTO creatSubscription = new CreatSubscriptionDTO();
        creatSubscription.setUserId(orderUpdate.getUserId());
        creatSubscription.setOrderId(orderUpdate.getOrderId());
        creatSubscription.setAppId(orderUpdate.getAppId());
        creatSubscription.setProductId(orderUpdate.getProductId());
        creatSubscription.setThirdSubscriptionId(paddleWebhookDTO.getSubscriptionId());
        String subscriptionId = subscriptionsService.creatSubscription(creatSubscription);
        // 用户信息
        // 查询订单信息获取产品id
        OrdersVO orderById = ordersService.getOrderById(orderUpdate.getOrderId());
        // 根据产品id查询产品信息
        ProductVO productVO = productApi.getProduct(orderById.getProductId()).getResult();
        // 获取产品平台
        Integer platform = productVO.getPlatform();

        // 修改订单状态
        UpdateUserForOrderDTO successDTO = new UpdateUserForOrderDTO();
        successDTO.setUserId(orderUpdate.getUserId());
        successDTO.setPayTpe(PaidTypeEnum.SUBSCRIBE.value());
        successDTO.setPlatform(platform);
        successDTO.setUserStatus(PDFOfficeUserStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
        successDTO.setEndDate(subscriptionsService.getSubscriptionBySubscriptionId(subscriptionId).getEndDate());
        userApi.updateUserForOrder(successDTO);
    }

    @Override
    public void subscriptionPaymentSucceeded(PaddleWebhookDTO paddleWebhookDTO) {
        log.info("paddle 订阅的支付订单已成功 paddle订单id：{}",paddleWebhookDTO.getCheckoutId());
        OrdersVO orderByTradeNo = ordersService.getOrderByTradeNo(paddleWebhookDTO.getPassthrough()).get(0);
        // 判断当前是否存在订阅
        SubscriptionsVO subscriptions = subscriptionsService.getSubscriptionsByProductId(orderByTradeNo.getUserId(), orderByTradeNo.getProductId());
        if (ObjectUtils.isEmpty(subscriptions)){
            throw new BackendRuntimeException(ErrorMessage.SUBSCRIPTION_NON_EXISTENT);
        }
        if (LocalDateTime.now().compareTo(MyDateUtils.dateToLocalDateTime(subscriptions.getEndDate())) > 0){
            throw new BackendRuntimeException(ErrorMessage.SUBSCRIPTION_NON_EXISTENT);
        }
        // 创建新订单
        CreatOrderManualDTO creatOrderManual = new CreatOrderManualDTO();
        BeanUtils.copyProperties(orderByTradeNo,creatOrderManual);
        creatOrderManual.setPaymentMethod(PaymentMethodEnum.PADDLE);
        // 设置实际扣费价格 有可能不对。。。
        creatOrderManual.setPrice(paddleWebhookDTO.getBalanceGross());
        creatOrderManual.setThirdTradeNo(paddleWebhookDTO.getCheckoutId());
        OrderUpdateVO orderManual = ordersService.creatOrderManual(creatOrderManual);
        // 创建新订阅
        CreatSubscriptionManualDTO subscriptionManual = new CreatSubscriptionManualDTO();
        BeanUtils.copyProperties(orderByTradeNo,orderManual);
        subscriptionManual.setProductId(orderByTradeNo.getProductId());
        subscriptionManual.setThirdSubscriptionId(paddleWebhookDTO.getSubscriptionId());
        subscriptionManual.setStatus(SubscriptionConstant.actived);
        subscriptionsService.creatSubscriptionManual(subscriptionManual);
    }
}
