package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.dto.UpdateUserForOrderDTO;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.*;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PaidTypeEnum;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.client.PaddleClient;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import cn.kdan.cloud.pdf.office.payment.mapper.SubscriptionsMapper;
import cn.kdan.cloud.pdf.office.payment.service.OrdersService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023-01-28
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class SubscriptionsServiceImpl extends ServiceImpl<SubscriptionsMapper, Subscription> implements SubscriptionsService {

    private final ProductApi productApi;

    private final PaddleClient paddleClient;

    private final UserApi userApi;

    private final OrdersService ordersService;

    private final RabbitTemplate rabbitTemplate;

    @Override
    public IPage<Subscription> page(Subscription query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }


    @Override
    public String creatSubscription(CreatSubscriptionDTO subscriptionInfo) {
        log.info("creatSubscription Info ：{}", subscriptionInfo);
        /* 获取产品的订阅周期时长 */
        // 当前时间
        LocalDateTime now = LocalDateTime.now();
        // 初始时间
        LocalDateTime originalTime = getOriginalTime(subscriptionInfo, now);
        // 获取结束时间
        LocalDateTime endTime = getEndTime(subscriptionInfo, originalTime);
        // 产生一条最新的订阅信息
        Subscription subscription = new Subscription();
        subscription.setProductId(subscriptionInfo.getProductId());
        subscription.setStatus(SubscriptionConstant.actived);
        subscription.setUserId(subscriptionInfo.getUserId());
        subscription.setStartDate(MyDateUtils.localDateTimeToDate(now));
        subscription.setEndDate(MyDateUtils.localDateTimeToDate(endTime));
        subscription.setThirdSubscriptionId(subscriptionInfo.getThirdSubscriptionId());
        subscription.setOrderId(subscriptionInfo.getOrderId());
        subscription.setAppId(subscriptionInfo.getAppId());
        this.baseMapper.insert(subscription);
        String subscriptionId = subscription.getId();
        // 发送消息
        SubscriptionUtil.subscriptionExpiredSendNews(rabbitTemplate, subscriptionId, endTime);
        return subscriptionId;
    }

    @Override
    public SubscriptionsVO getSubscriptions(String userId, PlatformEnum platform) {
        return this.baseMapper.selectNewestSubscription(userId, platform.getValue());
    }

    @Override
    public SubscriptionsVO getSubscriptionsByProductId(String userId, String productId) {
        return this.baseMapper.selectNewestSubscriptionByProductId(userId, productId);
    }

    @Override
    public String creatSubscriptionManual(CreatSubscriptionManualDTO subscriptionManual) {
        CreatSubscriptionDTO creatSubscriptionDTO = new CreatSubscriptionDTO();
        BeanUtils.copyProperties(subscriptionManual, creatSubscriptionDTO);
        return this.creatSubscription(creatSubscriptionDTO);
    }

    @Override
    public String giveSubscriptionManual(CreatSubscriptionManualDTO subscriptionManual) {
        String subscriptionId = this.creatSubscriptionManual(subscriptionManual);
        // 获取当前产品的所属平台
        Integer platform = productApi.getProduct(subscriptionManual.getProductId()).getResult().getPlatform();
        // TODO 判断当前用户的当前平台是否已在订阅中
        // 修改用户状态
        // 修改用户状态为试用中
        UpdateUserForOrderDTO successDTO = new UpdateUserForOrderDTO();
        successDTO.setUserId(subscriptionManual.getUserId());
        successDTO.setPayTpe(PaidTypeEnum.ON_TRIAL.value());
        successDTO.setUserStatus(Integer.valueOf(PDFOfficeUserStatusEnum.TRIAL.value()));
        successDTO.setPlatform(platform);
        userApi.updateUserForOrder(successDTO);
        return subscriptionId;
    }

    @Override
    public void chargebackSubscription(ChargebackOrderDTO chargebackOrderDTO) {
        Subscription subscription = this.baseMapper.selectById(chargebackOrderDTO.getSubscriptionId());
        String thirdSubscriptionId = subscription.getThirdSubscriptionId();
        // 更新paddle的订阅信息
        paddleClient.cancelSubscription(thirdSubscriptionId);
        // 修改订阅状态
//        Subscription subscriptions = new Subscription();
//        subscriptions.setId(chargebackOrderDTO.getSubscriptionId());
//        subscriptions.setStatus(SubscriptionConstant.refunded);
//        this.baseMapper.updateById(subscriptions);
    }

    @Override
    public SubscriptionSchemeVO getSubscriptionSchemeInfo(String userId, PlatformEnum platformEnum) {
        // 获取最新订阅信息
        SubscriptionsVO subscriptions = this.getSubscriptions(userId, platformEnum);
        // 获取产品信息
        ProductVO productVO = productApi.getProduct(subscriptions.getProductId()).getResult();
        // 获取该订阅关联的所有订单
        String orderId = subscriptions.getOrderId();
        String thirdTradeNo = ordersService.getOrderById(orderId).getThirdTradeNo();
        List<OrdersVO> ordersVOS = ordersService.getOrderByTradeNo(thirdTradeNo);
        SubscriptionSchemeVO subscriptionSchemeVO = new SubscriptionSchemeVO();
        SubscriptionInfoVO build = SubscriptionInfoVO.builder()
                .price(subscriptions.getPrice())
                .endDate(MyDateUtils.dateToLocalDateTime(subscriptions.getEndDate()))
                .paymentMethod(ordersVOS.get(0).getPayment())
                .productName(productVO.getProductName())
                .build();
        List<OrderInfoVO> ordersInfo = new ArrayList<>();
        ordersVOS.forEach(ordersVO -> {
            ordersInfo.add(OrderInfoVO.builder()
                    .date(MyDateUtils.dateToLocalDateTime(ordersVO.getCreatedAt()))
                    .scheme(productVO.getForPeriod())
                    .price(ordersVO.getPrice())
                    .platform(productVO.getPlatform())
                    .status(ordersVO.getStatus())
                    .productName(productVO.getProductName())
                    .build());
        });
        subscriptionSchemeVO.setSubscriptionInfo(build);
        subscriptionSchemeVO.setOrdersInfo(ordersInfo);
        return subscriptionSchemeVO;
    }

    @Override
    public SubscriptionsVO getSubscriptionBySubscriptionId(String subscriptionId) {
        return this.baseMapper.selectBySubscriptionId(subscriptionId);
    }


    /**
     * 获取订阅结束时间
     *
     * @param subscriptionInfo subscriptionInfo
     * @param originalTime     开始时间
     * @return 结束时间
     */
    private LocalDateTime getEndTime(CreatSubscriptionDTO subscriptionInfo, LocalDateTime originalTime) {
        if (!ObjectUtils.isEmpty(subscriptionInfo.getTermOfValidity())) {
            return MyDateUtils.TimeAddition(originalTime, subscriptionInfo.getTermOfValidity());
        }
        ProductVO product = productApi.getProduct(subscriptionInfo.getProductId()).getResult();
        String forPeriod = product.getForPeriod();
        return MyDateUtils.TimeAddition(originalTime, forPeriod);
    }

    /**
     * 获取初始开始时间
     *
     * @param subscriptionInfo subscriptionInfo
     * @param now              now
     * @return LocalDateTime
     */
    private LocalDateTime getOriginalTime(CreatSubscriptionDTO subscriptionInfo, LocalDateTime now) {
        // 通过用户id和产品id获取最新订阅信息
        SubscriptionsVO newestSubscriptions = getSubscriptionsByProductId(subscriptionInfo.getUserId(), subscriptionInfo.getProductId());
        LocalDateTime originalTime;
        // 没有订阅数据
        if (ObjectUtils.isEmpty(newestSubscriptions)) {
            originalTime = now;
        } else {
            LocalDateTime localDateTime = MyDateUtils.dateToLocalDateTime(newestSubscriptions.getEndDate());
            // 当前时间大于最后结束时间
            if (now.compareTo(localDateTime) > 0) {
                originalTime = now;
            } else {
                originalTime = localDateTime;
            }
        }
        return originalTime;
    }
}
