package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.website.service.PaymentService;
import cn.kdan.cloud.pdf.office.website.service.ProductInfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/2/9
 * <p>
 * PDF OFFICE app端 Controller
 */
@RestController
@RequestMapping("/pdf/office/app")
@RequiredArgsConstructor
public class PdfOfficeAppController {

    private final PaymentService paymentService;

    private final ProductInfoService productInfoService;

    /**
     * 获取订单记录
     *
     * @param platform platform
     * @return ResultMap
     */
    @GetMapping("/getOrderRecord")
    public ResultMap<SubscriptionSchemeVO> getOrderRecord(@RequestParam Integer platform) {
        return paymentService.getSubscriptionSchemeInfo(platform);
    }

    /**
     * 获取订阅结束时间
     *
     * @param platform 平台
     * @return LocalDateTime
     */
    @GetMapping("/getEndDate")
    public ResultMap<LocalDateTime> getEndDate(@RequestParam Integer platform) {
        return ResultMap.success(paymentService.getSubscriptionEndDate(platform));
    }

    /**
     * 获取新用户专用活动
     *
     * @param appId appId
     * @return 新用户专用活动
     */
    @GetMapping("/getNewUserProduct")
    public ResultMap<List<ProductVO>> getNewUserProduct(@Validated @NotNull(message = "appId不能为空") @RequestParam String appId){
        return productInfoService.getNewUserProduct(appId);
    }

}
