package cn.kdan.cloud.pdf.office.sso.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.authentication.OAuth2AuthenticationDetails;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;

@RestController
@RequestMapping("/user")
public class UserController {

    @Autowired
    private UserApi userApi;

    @Autowired
    private AuthService authService;

    @GetMapping("/me")
    public UserInfoVO me(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            {
                String userId = ((CustomUserDetails)((OAuth2Authentication)principal).getPrincipal()).getId();
                authService.refreshToken(principal);
                // 获取用户的基本信息
                UserInfoVO user = userApi.getInfoById(userId).getResult();
                user.setDigestPassword(null);
                return user;
            }
        }
        return null;
    }

    /**
     * 登出(登出设备)
     *
     * @param deviceSign 设备号
     * @param appId appId
     * @param request 请求
     * @param principal 凭证
     * @return Boolean
     */
    @PostMapping("/logout")
    public ResultMap<Boolean> logout(@RequestParam String deviceSign, @RequestParam String appId, HttpServletRequest request, Principal principal) {
        String token = getToken(principal);
        String userId = getUserId(principal);
        authService.logout(token,userId,deviceSign,appId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 用户自己手动注销
     *
     * @param code
     * @return
     */
    @PostMapping("/logOffForUser")
    public ResultMap<Boolean> logOffForUser(@RequestParam String code, Principal principal) {
        String userId = getUserId(principal);
        userApi.logOffForUser(code,userId).getResult();
        UserVO userVO = userApi.getById(userId).getResult();
        authService.logOutAll(userVO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }

    private String getUserId(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((CustomUserDetails) (((OAuth2Authentication) principal).getPrincipal())).getId();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }

    private String getToken(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((OAuth2AuthenticationDetails) (((OAuth2Authentication) principal).getDetails())).getTokenValue();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }
}
