package cn.kdan.cloud.pdf.office.sso.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.LoginDeviceApi;
import cn.kdan.cloud.pdf.office.api.account.feign.OauthClientDetailsApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.vo.LoginDevice;
import cn.kdan.cloud.pdf.office.api.account.vo.OauthClientDetails;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.dto.UserResetPasswordDTO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.RedisUtils;
import cn.kdan.cloud.pdf.office.common.vo.TokenVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.sso.constant.AuthConstant;
import cn.kdan.cloud.pdf.office.sso.enums.VerifyTypeEnum;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import cn.kdan.cloud.pdf.office.sso.utils.RSAUtils;
import cn.kdan.cloud.pdf.office.sso.utils.TokenUtils;
import com.alibaba.nacos.common.utils.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.context.annotation.Lazy;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2RefreshToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.authentication.OAuth2AuthenticationDetails;
import org.springframework.security.oauth2.provider.endpoint.AuthorizationEndpoint;
import org.springframework.security.oauth2.provider.endpoint.TokenEndpoint;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.bind.support.SimpleSessionStatus;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.*;
import java.util.stream.Collectors;

@Service
@RefreshScope
public class AuthServiceImpl implements AuthService {

    private final Logger logger = LoggerFactory.getLogger(AuthServiceImpl.class);

    @Autowired
    private RedisUtils<String, String> redisUtils;


    @Autowired
    private UserApi userApi;
    @Autowired
    private LoginDeviceApi loginDeviceApi;
    @Autowired
    private OauthClientDetailsApi oauthClientDetailsApi;

    @Autowired
    private AuthorizationEndpoint authorizationEndpoint;

    @Autowired
    private TokenEndpoint tokenEndpoint;
    @Autowired
    private EmailApi emailApi;


    @Value("${security.oauth2.client.user-authorization-uri}")
    private String authorizeUrl;

    @Value("${security.oauth2.client.access-token-uri}")
    private String tokenUri;
    @Value("${security.oauth2.client.client-id}")
    private String clientId;

    @Override
    public String getAuthorizeCode(String userId,String clientId, HttpServletRequest request) {
        //获取用户的基本信息
        UserVO userVO = userApi.getById(userId).getResult();
        Principal principal = request.getUserPrincipal();
        Map<String, Object> model = new HashMap<>();
        Map<String,String> params = new HashMap<>();
        params.put("response_type","code");
        params.put("client_id",clientId);
        params.put("redirect_uri",AuthConstant.DEFAULT_REDIRECT_URI);
        SessionStatus status = new SimpleSessionStatus();
        ModelAndView authorizeMV = authorizationEndpoint.authorize(model, params, status, principal);
        RedirectView view = (RedirectView) authorizeMV.getView();
        if(view != null && StringUtils.isNotEmpty(view.getUrl())) {
            return view.getUrl().substring(view.getUrl().indexOf("code=")+5);
        } else {
            return null;
        }
    }

    @Override
    public TokenVO getToken(String code, String clientId, HttpServletRequest request) {
        Map<String,String> params = new HashMap<>();
        params.put("grant_type","authorization_code");
        params.put("code",code);
        params.put("client_id",clientId);
        //获取用户服务节点信息
        params.put("redirect_uri",AuthConstant.DEFAULT_REDIRECT_URI);
        params.put("client_secret","");
        User principal = new User(clientId,"", Collections.emptySet());
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(principal, null, Collections.emptySet());
        try {
            ResponseEntity<OAuth2AccessToken> oAuth2AccessTokenResponseEntity = tokenEndpoint.postAccessToken(authenticationToken, params);
            return convert(oAuth2AccessTokenResponseEntity.getBody());
        } catch (HttpRequestMethodNotSupportedException e) {
            logger.error(e.getMessage(),e);
        }
        return null;
    }

    @Override
    public TokenVO  emailLogin(String email, String password, String appId, String platformType, String deviceSign,HttpServletRequest request) {
        //如果是管理平台就去查管理平台的用户表
        UserVO userVO = userApi.getByAppAccount(email,appId,platformType).getResult();
        //检查用户存在
        checkUser(userVO);
        checkPassword(password, userVO.getDigestPassword());
        //检查邮件验证码
        // checkEmailCodeValid(EmailCodeTypeEnum.LOGIN,email,code);
        // 检查设备是否达到上限
        checkLoginDeviceNum(userVO.getId(), deviceSign, appId);
        TokenVO vo = getTokenByUser(password, userVO);
        //关联设备登录
        relateTokenAndDevice(vo.getAccess_token(), userVO.getId(), deviceSign, appId);
        return vo;
    }

    private void checkPassword (String userPassword,String realPassword){
        realPassword =  new RSAUtils().decrypt(realPassword);
        if(!userPassword.equals(realPassword)){
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_PASSWORD_ERROR);
        }
    }

    @Override
    public TokenVO emailRegister(UserRegisterDTO userRegisterDTO) {
        validUserRegisterParam(userRegisterDTO);
        ResultMap<Boolean> resultMap = userApi.register(userRegisterDTO);
        if(resultMap.getCode() == CommonConstant.SUCCESS){
            //如果是管理平台就去查管理平台的用户表
            UserVO userVO = userApi.getByAppAccount(userRegisterDTO.getUsername(),
                    userRegisterDTO.getAppId(),userRegisterDTO.getPlatformType()).getResult();
            //生成token
            TokenVO vo = getTokenByUser(CommonUtils.generateDateId(), userVO);
            //关联设备登录
            relateTokenAndDevice(vo.getAccess_token(), userVO.getId(), userRegisterDTO.getDeviceSign(), userRegisterDTO.getAppId());
            return vo;
        }
        throw new BackendRuntimeException(AuthConstant.EMAIL_REGISTER_ERROR);
    }

    @Override
    public void resetPassword(UserResetPasswordDTO userResetPasswordDTO) {
        //检查验证码
        checkEmailCodeValid(EmailCodeTypeEnum.USER_RESET_PASSWORD,userResetPasswordDTO.getAccount(),userResetPasswordDTO.getVerifyCode(), userResetPasswordDTO.getAppId());
        if(!userResetPasswordDTO.getFirstPassword().equals(userResetPasswordDTO.getSecondPassword())){
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_PASSWORD_NOT_INCONSISTENT);
        }
        //检查密码长度
        if(userResetPasswordDTO.getFirstPassword().length()>6&&userResetPasswordDTO.getFirstPassword().length()<6){
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_PASSWORD_SIZE_MIN);
        }
        if(userResetPasswordDTO.getFirstPassword().length()>6&&userResetPasswordDTO.getFirstPassword().length()>24){
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_PASSWORD_SIZE_MAX);
        }
        UserVO userVO = userApi.getByAppAccount(userResetPasswordDTO.getAccount(),userResetPasswordDTO.getAppId(),userResetPasswordDTO.getPlatformType()).getResult();
        //修改密码
        userVO.setUpdatedAt(new Date());
        userVO.setDigestPassword(new RSAUtils().encrypt(userResetPasswordDTO.getFirstPassword()));
        userApi.updateUser(userVO);
        logOutAll(userVO);
    }

    @Override
    public void logOutAll(UserVO userVO) {
        List<LoginDevice> list = loginDeviceApi.getByUserId(userVO.getId()).getResult();
        list.forEach( item ->{
                    String token = redisUtils.hget(userVO.getAppId() + AuthConstant.DEVICE_LOGIN_TOKEN_KEY,
                            userVO.getId() + CommonConstant.STRIKE_THROUGH + item.getUniqueSn());
                    logout(token, userVO.getId(),item.getUniqueSn(), userVO.getAppId());
                }
        );
    }



    @Override
    public void validUserRegisterParam(UserRegisterDTO param) {
        //邮箱是否被注册
        if (!ObjectUtils.isEmpty(userApi.getByAppAccount(param.getUsername(),param.getAppId(),param.getPlatformType()).getResult())) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_EMAIL_EXIST);
        }
        //验证邮箱格式
        String emailRegex = CommonConstant.emailRegex;
        if (!param.getUsername().matches(emailRegex)) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_EMAIL_INVALID);
        }
    }

    @Override
    public void validUserLoginParam(UserRegisterDTO param) {
        //邮箱是否被注册
        if (ObjectUtils.isEmpty(userApi.getByAppAccount(param.getUsername(),param.getAppId(),param.getPlatformType()).getResult())) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_EMAIL_NOT_REGISTER);
        }
        //验证邮箱格式
        String emailRegex = CommonConstant.emailRegex;
        if (!param.getUsername().matches(emailRegex)) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_EMAIL_INVALID);
        }
    }

    @Override
    public boolean getVerifyCode(EmailCodeTypeEnum action, VerifyTypeEnum type, String receiver, String appId) {
        boolean flag = false;
        if (redisUtils.hexists(CommonConstant.VERIFY_CODE_TIME_OUT + action.value() + CommonConstant.STRIKE_THROUGH + appId, receiver)) {
            throw new BackendRuntimeException(AuthConstant.VERIFY_CODE_SEND_TOO_QUICKLY);
        }
        String code = CommonUtils.generateVerifyCode();
        switch (type) {
            //发送手机验证码
            case PHONE:
                //忘记密码时手机验证 发送手机短信
                break;
            //发送邮件
            case EMAIL:
                //创建账号
                EmailSendBO bo = new EmailSendBO();
                bo.setToEmail(receiver);
                handleEmailSendBO(action,bo,code);
                if(emailApi.sendEmail(bo).getCode() == HttpStatus.OK.value()){
                    flag = true;
                };
                break;
            default:
                break;
        }
        //存入缓存
        if (flag) {
            redisUtils.hset(CommonConstant.EMAIL_VERIFY_CODE_KEY + action.value() + CommonConstant.STRIKE_THROUGH + appId, receiver, code, AuthConstant.VERIFY_CODE_KEY_EXPIRE_TIME);
            //再次发送计时
            redisUtils.hset(CommonConstant.VERIFY_CODE_TIME_OUT + action.value() + CommonConstant.STRIKE_THROUGH + appId , receiver, "1", AuthConstant.VERIFY_CODE_KEY_RESEND_TIME);
        }
        return flag;
    }

    void handleEmailSendBO (EmailCodeTypeEnum action,EmailSendBO bo,String code) {
        //设置事件
        bo.setUseEvent(action.value());
        //设置title
        Map<String,String> titleMap = new HashMap<>();
        //设置内容
        Map<String,String> contentMap = new HashMap<>();
        contentMap.put("@code@",code);
        //注册
        if(action.value().equals(EmailCodeTypeEnum.USER_REGISTER.value())){
        }
        bo.setSendTitleContent(titleMap);
        //设置内容
        bo.setSendContent(contentMap);
    }

    @Resource
    @Lazy
    private TokenStore tokenStore;

    @Override
    public void logout(String token, String userId, String deviceSign, String appId) {
        OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(token);
        if (oAuth2AccessToken != null) {
            OAuth2RefreshToken oAuth2RefreshToken = oAuth2AccessToken.getRefreshToken();
            //从tokenStore中移除token
            tokenStore.removeAccessToken(oAuth2AccessToken);
            tokenStore.removeRefreshToken(oAuth2RefreshToken);
            tokenStore.removeAccessTokenUsingRefreshToken(oAuth2RefreshToken);
        }
        deleteTokenAndDevice(userId, deviceSign, appId);
    }

    /**
     * 判断用户是否存在
     *
     * @param userVO
     */
    private void checkUser(UserVO userVO) {
        if (ObjectUtils.isEmpty(userVO)) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_EMAIL_NOT_REGISTER);
        }
        //注销中的用户无法登陆
        if(userVO.getValidFlag().equals(PDFOfficeUserStatusEnum.LOGGED_OUT_ING.value().toString())){
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_EMAIL_LOGOUT_ING);
        }
    }

    /**
     * 检查设备是否达到上限
     * @param userId 用户id
     * @param deviceSign 设备
     */
    private void checkLoginDeviceNum(String userId, String deviceSign, String appId) {
        //根据userId查询如果list>2，并且设备号不属于其中的某一个
        List<LoginDevice> list = loginDeviceApi.getByUserId(userId).getResult();
        //验证token有效性，如果token已经过期则删除对应登录设备信息
        list.forEach(item->{
            String token = redisUtils.hget(appId + AuthConstant.DEVICE_LOGIN_TOKEN_KEY, userId + CommonConstant.STRIKE_THROUGH + deviceSign);
            OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(token);
            //如果token过期了，删除设备登录记录
            if(ObjectUtils.isEmpty(oAuth2AccessToken)||oAuth2AccessToken.isExpired()){
               loginDeviceApi.delete(deviceSign,userId);
            }
        });
        //查询经过处理后的设备记录
        List<LoginDevice> finalList = loginDeviceApi.getByUserId(userId).getResult();
        List<String> snList = finalList.stream().map(LoginDevice::getUniqueSn).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(finalList)&&finalList.size() >= 2&&!snList.contains(deviceSign)) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_DEVICE_NUM_MAX);
        }
    }

    /**
     * 校验邮件验证码
     *
     * @param type    邮件验证码类型
     * @param account 用户
     * @param code    验证码
     */
    private void checkEmailCodeValid(EmailCodeTypeEnum type, String account, String code, String appId) {
        //获取用户存在redis中的登录邮箱验证码
        String captchaCode = redisUtils.hget(CommonConstant.EMAIL_VERIFY_CODE_KEY + type.value() + CommonConstant.STRIKE_THROUGH + appId, account);
        if(StringUtils.isNotEmpty(code) && code.equals(captchaCode)){
            //验证通过删除
            redisUtils.hdel(CommonConstant.EMAIL_VERIFY_CODE_KEY + type.value() + CommonConstant.STRIKE_THROUGH + appId, account);
        }else{
            throw new BackendRuntimeException(CommonConstant.EMAIL_VERIFY_CODE_KEY_ERROR);
        }
    }

    /**
     * 判断邮件验证码是否存在
     *
     * @param type 邮件验证码类型 login  user_register
     * @param account 用户
     * @param code 验证码
     */
    @Override
    public void isEmailCodeValid(EmailCodeTypeEnum type, String account, String code, String appId) {
        //获取用户存在redis中的登录邮箱验证码
        String captchaCode = redisUtils.hget(AuthConstant.EMAIL_VERIFY_CODE_KEY + type.value() + CommonConstant.STRIKE_THROUGH + appId, account);
        if(StringUtils.isNotEmpty(code) && !code.equals(captchaCode)){
            throw new BackendRuntimeException(AuthConstant.EMAIL_VERIFY_CODE_KEY_ERROR);
        }
    }

    private TokenVO getTokenByUser(String code, UserVO userVO) {
        Set<SimpleGrantedAuthority> authorities = new LinkedHashSet<>();
        String account = userVO.getId();
        CustomUserDetails principal = new CustomUserDetails(userVO.getId(),
                account,
                code,
                true,
                true,
                true,
                true,
                authorities);
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(principal, null, Collections.emptySet());
        OauthClientDetails clientDetails = oauthClientDetailsApi.getByClientId(clientId).getResult();
        if (clientDetails == null) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_PLEASE_ADD_AUTH_CONFIG);
        }
        String authCode = TokenUtils.getAuthorizeCode(clientDetails.getClientId(), authenticationToken);
        OAuth2AccessToken token = TokenUtils.getToken(authCode, clientId, clientDetails.getClientSecret());
        return TokenUtils.convert(token);
    }


    /**
     * 将设备的token存在redis ，登出时根据对应的规则去删除
     * @param token
     * @param userId
     * @param deviceSign
     * @param appId
     */
    private void relateTokenAndDevice(String token,String userId,String deviceSign,String appId) {
        LoginDevice loginDevice = loginDeviceApi.getByDeviceSignAndAppId(deviceSign,userId).getResult();
        if(ObjectUtils.isEmpty(loginDevice)){
            //如果账号在该设备没有登录过
            LoginDevice loginDeviceBo = new LoginDevice();
            loginDeviceBo.setId(CommonUtils.generateId());
            loginDeviceBo.setCreatedAt(new Date());
            loginDeviceBo.setUniqueSn(deviceSign);
            loginDeviceBo.setUserId(userId);
            loginDeviceBo.setAppId(appId);
            loginDeviceApi.create(loginDeviceBo);
        }
        redisUtils.hset(appId + AuthConstant.DEVICE_LOGIN_TOKEN_KEY, userId + CommonConstant.STRIKE_THROUGH + deviceSign, token);
    }

    private void deleteTokenAndDevice(String userId, String deviceSign, String appId) {
        // 登录设备表删除数据关联用户
        loginDeviceApi.delete(deviceSign, userId);
        redisUtils.hdel(appId + AuthConstant.DEVICE_LOGIN_TOKEN_KEY, userId + CommonConstant.STRIKE_THROUGH + deviceSign);
    }

    private TokenVO convert(OAuth2AccessToken token) {
        if (token == null) {
            return null;
        }
        TokenVO tokenVO = new TokenVO();
        tokenVO.setAccess_token(token.getValue());
        tokenVO.setExpires_in(token.getExpiresIn()+"");
        tokenVO.setToken_type(token.getTokenType());
        tokenVO.setRefresh_token(token.getRefreshToken().getValue());
        return tokenVO;
    }

    @Override
    public void refreshToken(Principal principal) {
        String token = ((OAuth2AuthenticationDetails)((OAuth2Authentication) principal).getDetails()).getTokenValue();
        Set<String> keySet = new HashSet<>();
        keySet.add(AuthConstant.ACCESS+token);
        keySet.add(AuthConstant.AUTH+token);
        keySet.add(AuthConstant.ACCESS_TO_REFRESH + token);
        long time = 30 * 24 * 60 * 60L;
        keySet.forEach(key -> redisUtils.expire(key, time));
    }


}
